'
' Copyright (c) 2015, MindFusion LLC - Bulgaria.
'

Imports System.Drawing.Drawing2D
Imports System.Diagnostics
Imports System.Runtime.InteropServices


Public Class MainForm

    Dim pcProcessor As PerformanceCounter
    Dim pcMemory As PerformanceCounter

    Dim cpuPointer As Pointer
    Dim ramPointer As Pointer
    Dim ramScale As OvalScale
    Dim cpuScale As OvalScale

    Dim cpuText As Text
    Dim roundness = 25

    Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        pcProcessor = Nothing
        pcMemory = Nothing

        cpuPointer = cpuGauge.GetElementByName("CpuPointer")
        ramPointer = ramGauge.GetElementByName("RamPointer")
        cpuScale = cpuGauge.GetElementByName("CpuScale")
        ramScale = ramGauge.GetElementByName("RamScale")

        ' set up ram gauge scale values
        ramScale.MinValue = 0
        Dim stat As New MemoryStatusEx
        GlobalMemoryStatusEx(stat)
        Dim ram As Single = CType(stat.TotalPhysical, Single)
        ramScale.MaxValue = ram / 1024 / 1024

        ' CPU usage label
        cpuText = New Text()
        cpuText.FontFamily = "Consolas"
        cpuText.FontSize = New Length(20.0F, LengthType.Relative)
        cpuText.FontStyle = FontStyle.Bold
        cpuText.Foreground = Color.FromArgb(&HFF, &H7F, &HFF, &H0)
        cpuText.LineAlignment = StringAlignment.Center
        cpuText.Alignment = StringAlignment.Center
        cpuText.Content = "0%"
        cpuText.Margin = New Thickness(0, 0.3F, 0, 0)
        cpuScale.ScaleChildren.Add(cpuText)

        ' Needed to make the back color transparent
        ramUsed.Parent = ramGauge
        ramUsed.BackColor = Color.Transparent
        ramFree.Parent = ramGauge
        ramFree.BackColor = Color.Transparent
        ramUsedMB.Parent = ramGauge
        ramUsedMB.BackColor = Color.Transparent

        ' Add some event handlers
        AddHandler cpuScale.QueryLabelValue, AddressOf cpuScale_QueryLabelValue
        AddHandler ramScale.QueryLabelValue, AddressOf ramScale_QueryLabelValue
        AddHandler cpuPointer.ValueChanged, AddressOf cpuPointer_ValueChanged
        AddHandler cpuGauge.PrepaintForeground, AddressOf cpuGauge_PrepaintForeground
        AddHandler cpuGauge.PrepaintBackground, AddressOf cpuGauge_PrepaintBackground
        AddHandler ramGauge.PrepaintForeground, AddressOf ramGauge_PrepaintForeground
        AddHandler ramGauge.PrepaintBackground, AddressOf ramGauge_PrepaintBackground

    End Sub

    Private Sub MainForm_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        'Init performance counters and start the timer
        pcProcessor = New PerformanceCounter()
        pcProcessor.CategoryName = "Processor"
        pcProcessor.CounterName = "% Processor Time"
        pcProcessor.InstanceName = "_Total"

        pcMemory = New PerformanceCounter("Memory", "Available MBytes")

        timer.Enabled = True
    End Sub

    Private Sub timer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles timer.Tick
        'Update values
        cpuPointer.Value = pcProcessor.NextValue()
        ramPointer.Value = pcMemory.NextValue()
        Dim stat As New MemoryStatusEx()
        GlobalMemoryStatusEx(stat)
        Dim percent As Single = stat.MemoryLoad
        Dim free As Single = stat.AvailablePhysical / 1024 / 1024
        Dim used As Single = (stat.TotalPhysical - stat.AvailablePhysical) / 1024 / 1024
        ramUsed.Text = String.Format("RAM used: {0:0}%", percent)
        ramFree.Text = String.Format("Free: {0:0}MB", free)
        ramUsedMB.Text = String.Format("Used: {0:0}MB", used)
    End Sub

    Sub cpuScale_QueryLabelValue(ByVal sender As Object, ByVal e As QueryLabelValueEventArgs)
        e.NewValue = String.Format("{0:0}%", e.CalculatedLabelValue)
    End Sub

    Sub ramScale_QueryLabelValue(ByVal sender As Object, ByVal e As QueryLabelValueEventArgs)
        e.NewValue = String.Format("{0:0}MB", e.CalculatedLabelValue)
    End Sub

    Sub cpuPointer_ValueChanged(ByVal sender As Object, ByVal e As ValueChangedEventArgs(Of Single))
        cpuText.Content = String.Format("{0:0}%", e.NewValue)
    End Sub

    Sub ramGauge_PrepaintBackground(ByVal sender As Object, ByVal e As PrepaintEventArgs)
        e.CancelDefaultPainting = True
        PaintCommonBackgroundElements(e)
    End Sub

    Sub ramGauge_PrepaintForeground(ByVal sender As Object, ByVal e As PrepaintEventArgs)
        e.CancelDefaultPainting = True
        PaintCommonForegroundElements(e)
    End Sub

    Sub cpuGauge_PrepaintBackground(ByVal sender As Object, ByVal e As PrepaintEventArgs)
        e.CancelDefaultPainting = True
        PaintCommonBackgroundElements(e)

        ' CPU usage label background
        Dim textRect As New RoundRectangle()
        textRect.Fill = e.CreatePathGradient( _
            New PointF(0.5F, 0.3F), _
            0.0F, Color.FromArgb(&H80, &HC0, &HC0, &HC0), _
            0.2F, Color.FromArgb(&H9A, &H64, &H64, &H64), _
            0.6F, Color.FromArgb(&HBE, &H0, &H0, &H0), _
            1.0F, Color.FromArgb(&HFF, &H0, &H0, &H0))
        Dim strokeBrush As New MindFusion.Drawing.LinearGradientBrush( _
            Color.FromArgb(&HFF, &HA7, &HA7, &HA7), _
            Color.FromArgb(&HFF, &HFE, &HFE, &HFE), 130.0F)
        textRect.Stroke = New MindFusion.Drawing.Pen(strokeBrush, 0.01F)
        textRect.Margin = New Thickness(0.37F, 0.55F, 0.37F, 0.32F)
        e.PaintVisualElement(textRect, e.Element.RenderSize)
    End Sub

    Sub cpuGauge_PrepaintForeground(ByVal sender As Object, ByVal e As PrepaintEventArgs)
        e.CancelDefaultPainting = True
        PaintCommonForegroundElements(e)
    End Sub

    Sub PaintCommonBackgroundElements(ByVal e As PrepaintEventArgs)
        ' Layer 1
        Dim rect As New RoundRectangle()
        rect.Margin = New Thickness(-0.11F, 0.2F, -0.11F, 0.08F)
        rect.Fill = e.CreateLinearGradient(45.0F, _
            0.0F, Color.FromArgb(&HFF, &H32, &H32, &H32), _
            0.2F, Color.FromArgb(&HFF, &H32, &H32, &H32), _
            0.8F, Color.FromArgb(&HFF, &H64, &H64, &H64), _
            1.0F, Color.FromArgb(&HFF, &H64, &H64, &H64))
        rect.Stroke = New MindFusion.Drawing.Pen((Color.Transparent), 0)
        rect.Roundness = roundness
        e.PaintVisualElement(rect, e.Element.RenderSize)

        ' Layer 2
        rect.Margin = New Thickness(-0.1F, 0.21F, -0.1F, 0.09F)
        rect.Fill = e.CreateLinearGradient(45.0F, _
            0.0F, Color.FromArgb(&HFF, &H64, &H64, &H64), _
            0.2F, Color.FromArgb(&HFF, &H64, &H64, &H64), _
            0.8F, Color.FromArgb(&HFF, &H32, &H32, &H32), _
            1.0F, Color.FromArgb(&HFF, &H32, &H32, &H32))
        rect.Stroke = New MindFusion.Drawing.Pen((Color.Transparent), 0)
        rect.Roundness = roundness
        e.PaintVisualElement(rect, e.Element.RenderSize)

        ' Layer 3
        rect.Margin = New Thickness(-0.09F, 0.22F, -0.09F, 0.1F)
        rect.Fill = e.CreateLinearGradient(45.0F, _
            0.0F, Color.FromArgb(&HFF, &H32, &H32, &H32), _
            0.2F, Color.FromArgb(&HFF, &H32, &H32, &H32), _
            0.8F, Color.FromArgb(&HFF, &H64, &H64, &H64), _
            1.0F, Color.FromArgb(&HFF, &H64, &H64, &H64))
        rect.Stroke = New MindFusion.Drawing.Pen((Color.Transparent), 0)
        rect.Roundness = roundness
        e.PaintVisualElement(rect, e.Element.RenderSize)
    End Sub

    Sub PaintCommonForegroundElements(ByVal e As PrepaintEventArgs)
        ' Layer 1
        Dim rect As New RoundRectangle()
        rect.Margin = New Thickness(-0.09F, 0.22F, -0.09F, 0.1F)
        rect.Fill = e.CreateLinearGradient(45.0F, _
            0.0F, Color.FromArgb(&H64, &HFF, &HFF, &HFF), _
            0.2F, Color.FromArgb(&H32, &HFF, &HFF, &HFF), _
            0.8F, Color.FromArgb(&H10, &HFF, &HFF, &HFF), _
            1.0F, Color.FromArgb(&H0, &HFF, &HFF, &HFF))
        rect.Roundness = roundness
        e.PaintVisualElement(rect, e.Element.RenderSize)

        Dim bounds As New RectangleF(0, 0, 1, 1)
        bounds.Inflate(0, -0.2F)

        ' Reflections
        Dim areaPath As New GraphicsPath()
        areaPath.AddEllipse(bounds)

        Dim area1Fill As New MindFusion.Drawing.PathGradientBrush( _
            Color.FromArgb(153, Color.White), Color.Transparent)
        area1Fill.Path = areaPath

        Dim area2Fill As New MindFusion.Drawing.PathGradientBrush( _
            Color.FromArgb(32, Color.White), Color.Transparent)
        area2Fill.Path = areaPath

        Dim area1 As New ArcArea()
        area1.StartAngle = 160
        area1.EndAngle = -20
        area1.Margin = New Thickness(0.03F)
        area1.Fill = area1Fill

        Dim area2 As New ArcArea()
        area2.StartAngle = 140
        area2.EndAngle = -40
        area2.Margin = New Thickness(0.03F)
        area2.Fill = area2Fill

        e.PaintVisualElement(area1, e.Element.RenderSize)
        e.PaintVisualElement(area2, e.Element.RenderSize)

        areaPath.Dispose()
    End Sub

    <StructLayout(LayoutKind.Sequential)> _
    Private Class MemoryStatusEx
        Public Length As UInt32
        Public MemoryLoad As UInt32
        Public TotalPhysical As ULong
        Public AvailablePhysical As ULong
        Public TotalPageFile As ULong
        Public AvailablePageFile As ULong
        Public TotalVirtual As ULong
        Public AvailableVirtual As ULong
        Public AvailableExtendedVirtual As ULong
        Sub New()
            Me.Length = CUInt(Marshal.SizeOf(Me))
        End Sub
    End Class


    <DllImport("kernel32.dll", CharSet:=CharSet.Auto, SetLastError:=True)> _
    Private Shared Function GlobalMemoryStatusEx(<[In](), Out()> ByVal lpBuffer As MemoryStatusEx) As <MarshalAs(UnmanagedType.Bool)> Boolean
    End Function
End Class
